package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.mark.setting.MarkGroupDto;
import com.qmth.teachcloud.common.bean.dto.mark.setting.MarkGroupTaskDto;
import com.qmth.teachcloud.common.bean.dto.mark.setting.PictureConfig;
import com.qmth.teachcloud.common.bean.params.mark.setting.GroupPictureConfigParams;
import com.qmth.teachcloud.common.entity.MarkPaper;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.MarkPaperService;
import com.qmth.teachcloud.common.service.MarkQuestionService;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.mapper.MarkGroupMapper;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkUserClassService;
import com.qmth.teachcloud.mark.service.MarkUserGroupService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 评卷分组表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkGroupServiceImpl extends ServiceImpl<MarkGroupMapper, MarkGroup> implements MarkGroupService {

    @Resource
    MarkQuestionService markQuestionService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    MarkUserGroupService markUserGroupService;

    @Resource
    MarkUserClassService markUserClassService;

    @Override
    public MarkGroupTaskDto listGroupTaskByExamIdAndPaperNumber(Long examId, String paperNumber) {
        MarkGroupTaskDto markGroupTaskDto = new MarkGroupTaskDto();
        markGroupTaskDto.setExamId(examId);
        markGroupTaskDto.setPaperNumber(paperNumber);

        // 分组信息
        List<MarkGroup> groupList = listGroupByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(groupList)) {
            List<MarkGroupDto> markGroupDtoList = new ArrayList<>();
            for (MarkGroup markGroup : groupList) {
                MarkGroupDto markGroupDto = new MarkGroupDto();
                markGroupDto.setGroupNumber(markGroup.getNumber());
                markGroupDto.setDoubleRate(markGroup.getDoubleRate());
                markGroupDto.setArbitrateThreshold(markGroup.getArbitrateThreshold());
                markGroupDto.setScorePolicy(markGroup.getScorePolicy());
                if (StringUtils.isNotBlank(markGroup.getPicList())) {
                    markGroupDto.setPictureConfigs(JSON.parseArray(markGroup.getPicList(), PictureConfig.class));
                }
                // 评卷员
                markGroupDto.setMarkers(markUserGroupService.listGroupUserByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber()));
                // 题目
                markGroupDto.setQuestions(markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, paperNumber, markGroup.getNumber(), false));
                markGroupDto.setGroupQuestions(markQuestionService.assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, markGroup.getNumber()));
                markGroupDtoList.add(markGroupDto);
            }
            markGroupTaskDto.setGroups(markGroupDtoList);
        }
        // 分班阅参数
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        markGroupTaskDto.setOpenMarkClass(markPaper != null && markPaper.getOpenMarkClass());
        return markGroupTaskDto;
    }

    @Override
    public List<MarkGroup> listGroupByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkGroup::getExamId, examId)
                .eq(MarkGroup::getPaperNumber, paperNumber)
                .orderByAsc(MarkGroup::getNumber);
        return null;
    }

    @Transactional
    @Override
    public void deleteGroupByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer groupNumber) {

    }

    @Override
    public void updatePicListByExamIdAndPaperNumberAndGroupNumber(GroupPictureConfigParams groupPictureConfigParams) {
        List<PictureConfig> pictureConfigs = groupPictureConfigParams.getPictureConfigs();
        if (CollectionUtils.isEmpty(pictureConfigs)) {
            throw ExceptionResultEnum.ERROR.exception("没有评卷区数据");
        }
        UpdateWrapper<MarkGroup> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkGroup::getPicList, JSON.toJSONString(pictureConfigs))
                .eq(MarkGroup::getExamId, groupPictureConfigParams.getExamId())
                .eq(MarkGroup::getPaperNumber, groupPictureConfigParams.getPaperNumber())
                .eq(MarkGroup::getNumber, groupPictureConfigParams.getGroupNumber());
        this.update(updateWrapper);
    }

    @Transactional
    @Override
    public void updateOpenMarkClassByExamIdAndPaperNumber(Long examId, String paperNumber, Boolean openMarkClass) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getOpenMarkClass, openMarkClass)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);

        // 禁用，删除分班数据
        if (!openMarkClass.booleanValue()) {
            markUserClassService.deleteByExamIdAndPaperNumber(examId, paperNumber);
        }
    }
}
